package jp.sourceforge.freegantt.swing;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import jp.sourceforge.freegantt.data.Member;
import jp.sourceforge.freegantt.data.Project;
import jp.sourceforge.freegantt.data.Task;
import jp.sourceforge.freegantt.locale.Resource;
import jp.sourceforge.freegantt.util.CalendarUtil;

public class TaskListTableModel implements TableModel {
	
	public static int ADDITION_ROOM_COUNT = 20;
	String[] columnNames;
	public static String[] COLOMN_KEYS = {"", "name", "resouce", "period", "completion", "startDate"};

	List<TableModelListener> tableModelListeners = new ArrayList<TableModelListener>();
	Project project;
	
	public TaskListTableModel(Project project) {
		super();
		this.project = project;
		this.columnNames = new String[]{
				"", 
				Resource.get("listColumnName"), 
				Resource.get("listColumnResource"), 
				Resource.get("listColumnPeriod"), 
				Resource.get("listColumnCompletion"), 
				Resource.get("listColumnStartDate")				
		};
	}

	@Override
	public int getRowCount() {
		int size = ADDITION_ROOM_COUNT;
		for (Task task: project.getTasks()) {
			size = Math.max(task.getIndex() + ADDITION_ROOM_COUNT, size);
		}
		return size;
	}

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		if (columnIndex < 0 || columnIndex >= columnNames.length) return null;
		return columnNames[columnIndex];
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
		case 2:
			return Member.class;
		default:
			return String.class;
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		if (columnIndex == 0) return false;

		// TODO マイルストーンや親タスクでは自動的に決まる値を編集させてもしょうがないのでここでfalseにしたほうがよい
		
		return true;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (columnIndex == 0) return rowIndex + 1;
		
		Task task = project.findTask(rowIndex);
		if (task == null) return null;
		
		switch (columnIndex) {
		case 1:
			return task.getName();
		case 2:
			return task.getMember();
		case 3:
			return (task.getPeriod() == null) ? null : String.valueOf(task.getPeriod().intValue());
		case 4:
			if (task.getStartDate() == null) return null;
			if (task.isMilestone()) return null;
			return task.getCompletion();
		case 5:
			SimpleDateFormat dateFormat = new SimpleDateFormat(Resource.get("dateFormat"), Resource.getLocale());
			return (task.getStartDate() == null) ? null : dateFormat.format(task.getStartDate().getTime());
		}
		
		return null;
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		Task task = project.findTask(rowIndex);
		if (task == null) {
			task = new Task();
			task.setIndex(rowIndex);
			project.getTasks().add(task);
		}
		
		switch (columnIndex) {
		case 1:
			task.setName((String)aValue);
			break;
		case 2:
			task.setMember((Member)aValue);
			break;
		case 3:
			try {
				task.setPeriod((aValue == null || ((String)aValue).isEmpty()) ? null : Integer.parseInt((String)aValue));
			} catch (NumberFormatException ex) {
			}
			break;
		case 4:
			if (task.getStartDate() == null) break;
			if (task.isMilestone()) break;
			try {
				task.setCompletion((aValue == null || ((String)aValue).isEmpty()) ? 0 : Integer.parseInt((String)aValue));
			} catch (NumberFormatException ex) {
				task.setCompletion(0);
			}
			break;
		case 5:
			if (aValue == null || ((String)aValue).isEmpty()) {
				task.setStartDate(null);
				break;
			}
			Calendar startDate = CalendarUtil.parseDate((String)aValue, Resource.getLocale());
			if (startDate != null) {
				task.setStartDate(startDate);
				if (task.getPeriod() == null) task.setPeriod(0);
			}
			break;
		}
		
		project.update();
		
		for (TableModelListener listener: tableModelListeners) {
			TableModelEvent e = new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE);
			listener.tableChanged(e);
		}
	}

	@Override
	public void addTableModelListener(TableModelListener l) {
		tableModelListeners.add(l);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
		tableModelListeners.remove(l);
	}
	
	public void fireTableChanged() {
		for (TableModelListener listener: tableModelListeners) {
			TableModelEvent e = new TableModelEvent(this, 0, 0, 0, TableModelEvent.UPDATE);
			listener.tableChanged(e);
		}
	}
}
